--[[
	### ItemValue ###
	ItemValue StatSet Class
	
--]]

local class = select(2, UnitClass("player"))
if class ~= "DRUID" and class ~="MAGE" and class ~="PALADIN" and class ~="PRIEST" and class ~="SHAMAN" and class ~="WARLOCK" then return end

-- Libraries
local L = AceLibrary("AceLocale-2.2"):new("ItemValue_DrDamage")
local AceOO = AceLibrary("AceOO-2.0")
local ItemBonusLib = AceLibrary("ItemBonusLib-1.0")
local StatLogic = AceLibrary("StatLogic-1.0")

local class = select(2, UnitClass("player"))
local level = UnitLevel("player")

local Stats = {
	"STR", "AGI", "STA", "INT", "SPI",
	"DMG", "ARCANEDMG", "FIREDMG", "FROSTDMG", "HOLYDMG", "NATUREDMG", "SHADOWDMG",
	"HEAL", "MANAREG",
	"CR_SPELLCRIT", "CR_SPELLHIT", "SPELLPEN", "CR_SPELLHASTE",
	"ATTACKPOWER", "CR_CRIT", "CR_HIT",
}

local StatNames = {
	DMG					= L["Spelldamage"],
	ARCANEDMG			= L["Arcane Spelldamage"],
	FIREDMG				= L["Fire Spelldamage"],
	FROSTDMG			= L["Frost Spelldamage"],
	HOLYDMG				= L["Holy Spelldamage"],
	NATUREDMG			= L["Nature Spelldamage"],
	SHADOWDMG			= L["Shadow Spelldamage"],
	CR_SPELLCRIT		= L["Spell Crit Rating"],
	CR_SPELLHIT			= L["Spell Hit Rating"],
	SPELLPEN			= L["Spell Penetration"],
	CR_SPELLHASTE		= L["Spell Haste Rating"],
}

local DrDamageSet = AceOO.Class()

local db
function DrDamageSet.LoadSets()
	db = ItemValue:AcquireDBNamespace("DrDamage")
	
	for SetName in pairs(db.profile) do
		if not ItemValue.SetByName[SetName] then
			ItemValue:RegisterSet(DrDamageSet:new(SetName))
		end
	end
end

local validate_SpellName = function(v)
	if (v and type(v) == "string" and v:len() > 3) then
		return true
	else
		ItemValue:Print(L["Error: Invalid spell name!"])
		return false
	end
end

local validate_SpellRank = function(n)
	if (n and tonumber(n) and tonumber(n) > 0) then
		return true
	else
		ItemValue:Print(L["Error: Invalid spell rank!"])
		return false
	end
end

function DrDamageSet.prototype:init(Name, Enabled, Spell, SpellRank, UpdateMode, SpellData)
    DrDamageSet.super.prototype.init(self)
	
	if not db.profile[Name] then
		db.profile[Name] = {
			Enabled = (Enabled == nil) or Enabled,
			Spell = Spell or Name,
			SpellRank = SpellRank,
			UpdateMode = UpdateMode or L["Every Calculation"],
			StatValues = StatValues,
		}
	end
	
	self.Name = Name
	self.Enabled = db.profile[Name].Enabled
	self.Spell = db.profile[Name].Spell
	self.SpellRank = db.profile[Name].SpellRank
	self.UpdateMode = db.profile[Name].UpdateMode
	self.StatValues = db.profile[Name].StatValues
	
	
	self.Options = {
		type = "group",
		name = Name,
		desc = string.format(L["Settings for Set \"%s\"."], Name),
		args = {
			SpellName = {
				order = 10,
				type  = "text",
				name  = L["Spell"],
				desc  = L["Spell"],
				get   = function() return self.Spell end,
				set   = function(v) self.Spell = v db.profile[Name].Spell = v self:UpdateValues() ItemValue:ClearCache() end,
				usage = "<Spell Name>",
				validate = validate_SpellName,
			},
			SpellRank = {
				order = 11,
				type  = "text",
				name  = L["Spell Rank"],
				desc  = L["Spell Rank"],
				get   = function() return self.SpellRank end,
				set   = function(v) v = tonumber(v) self.SpellRank = v db.profile[Name].SpellRank = v self:UpdateValues() ItemValue:ClearCache() end,
				usage = "<Spell Rank>",
				validate = validate_SpellRank,
			},
			UpdateMode = {
				order = 12,
				type  = "text",
				name  = L["Update Mode"],
				desc  = L["Update Mode"],
				get   = function() return self.UpdateMode end,
				set   = function(v) self.UpdateMode = v db.profile[Name].UpdateMode = v self:UpdateValues() ItemValue:ClearCache() end,
				validate = {L["Every Calculation"], L["Saved Values"]}
			},
			UpdateValues = {
				order = 13,
				type  = "execute",
				name  = L["Update Values"],
				desc  = L["Updates saved values."],
				func  = function() self:UpdateValues() self:CalcValues() end,
				disabled = function() return self.UpdateMode ~= L["Saved Values"] end,
			},
			CopyToStatSet = {
				order = 14,
				type  = "text",
				name  = L["Copy to StatSet"],
				desc  = L["Creates a StatSet with identically StatValues."],
				usage = L["<Name>"],
				get   = false,
				set   = function(v)
					self:UpdateValues()
					local StatValues = {}
					for StatName, StatValue in pairs(self.StatValues) do
						StatValues[StatName] = StatValue
					end
					ItemValue.RegisterSet(ItemValue.classes.StatSet:new(v, self.Enabled, StatValues))
				end,
				validate = function(v) return tostring(v) and string.len(v) > 0 end,
			},
			Values = {
				order = 15,
				type  = "group",
				name  = L["Values"],
				desc  = L["Displays current stat value data."],
				args  = {},
			},
		},
	}
	
	local get = function(k)
		if not self.StatValues then self:UpdateValues() end
		return self.StatValues[k] or 0
	end
	local set = function() end
	local hidden = function(k)
		if not (self.StatValues and self.StatValues[k]) or self.StatValues[k] == 0 then
			return true
		end
	end
	
	for i, iblname in pairs(Stats) do
		self.Options.args.Values.args[iblname] = {
			order	= i,
			type	= "text",
			name	= StatNames[iblname] or ItemBonusLib:GetBonusFriendlyName(iblname),
			desc	= StatNames[iblname] or ItemBonusLib:GetBonusFriendlyName(iblname),
			get		= get,
			set		= set,
			usage   = "<Read-only!>",
			hidden  = hidden,
			passValue = iblname,
		}
	end
end

function DrDamageSet.prototype:tostring()
	return "ItemValue DrDamageSet " .. self.Name
end

function DrDamageSet.prototype:OnDelete()
	db.profile[self.Name] = nil
end

function DrDamageSet.prototype:GetEquipValue(eq)
	if not (self.Spell and self.SpellRank) then return 0 end
	
	self:UpdateValues()
	
	local Points = 0
	
	if eq then
		for StatName, StatValue in pairs(self.StatValues) do
			if eq[StatName] and StatValue then
				Points = Points + (eq[StatName] * StatValue)
			end
		end
	end
	
	return Points
end

function DrDamageSet.prototype:GetItemValue(ItemString)
	return self:GetEquipValue(ItemBonusLib:ScanItem(ItemString, true))
end

function DrDamageSet.prototype:UpdateValues()
	if self.UpdateMode == L["Saved Values"] then
		if db.profile[self.Name].StatValues then
			self.StatValues = db.profile[self.Name].StatValues
		else
			db.profile[self.Name].StatValues = {}
			self.StatValues = db.profile[self.Name].StatValues
		end
	else
		if self.StatValues then
			db.profile[self.Name].StatValues = nil
		else
			self.StatValues = {}
		end
		
		self:CalcValues()
	end
end

-- From DrDamage
local function MatchData(data, ... )
	if not (data and ...) then
		return false
	end
	
	if type(data) == "table" then
		for _, name in ipairs( data ) do
			for i=1, select('#', ...) do
				if name == select(i, ...) then
					return true
				end
			end
		end
	else
		for i=1, select('#', ...) do
			if data == select(i, ...) then
				return true
			end
		end
	end
	
	return false
end

function DrDamageSet.prototype:CalcValues()
	local Spell, SpellRank = self.Spell, self.SpellRank
	if not (validate_SpellName(Spell) and validate_SpellRank(SpellRank)) then return end

	local SpellLink = GetSpellLink(("%s(Rank %d)"):format(Spell, SpellRank))
	if not SpellLink then SpellLink = GetSpellLink(spellName) end
	if not SpellLink then return end
	local SpellId = tonumber(SpellLink:match("spell:(%d+)"))

	local TempSpellData = select(2, DrDamage:CasterCalc(Spell, SpellRank, Spell, true))
	if not TempSpellData then
		ItemValue:Print(string.format(L["Error: Couldn't get informations about |cffffff78%s|r from DrDamage!"], Spell))
		return
	end
	
	-- Spell Haste Value Calculation
	local baseCastTime = (select(7,GetSpellInfo(SpellLink)) or 1500) / 1000
	local NextHasteRating = 0
	if baseCastTime then
		local baseHaste = (ItemValue.EquipStats.CR_SPELLHASTE or 0)
		
		-- Hasted GCD is limited to 1 by 2.4
		local oldCastTime = math.max(baseCastTime / (1 + (baseHaste/1570)), 1)
		local newCastTime = math.max(baseCastTime / (1 + ((baseHaste+1)/1570)), 1)
		
		local hasteMod = newCastTime/oldCastTime
		NextHasteRating = TempSpellData.AvgTotal/hasteMod-TempSpellData.AvgTotal
	end
	
	local NextSpellCrit = TempSpellData.NextCrit or 0
	local NextSpellHit = TempSpellData.NextHit or 0
	local NextDamage = TempSpellData.NextTenDmg * 0.1 * StatLogic:GetStatMod("MOD_SPELL_DMG") or 0
	
	for k in pairs(self.StatValues) do
		self.StatValues[k] = nil
	end
	
	local TableSpell0 = DrDamage.spellInfo[Spell][0]
	if type(TableSpell0) == "function" then
	   TableSpell0 = TableSpell0()
	end
	local SpellSchool = TableSpell0.School

	self.StatValues.CR_SPELLCRIT = NextSpellCrit * DrDamage:GetRating("Crit", 1, true)
	self.StatValues.CR_SPELLHIT = NextSpellHit * DrDamage:GetRating("Hit", 1, true)
	self.StatValues.CR_SPELLHASTE = NextHasteRating
	
	self.StatValues.FIREDMG = 0
	self.StatValues.FROSTDMG = 0
	self.StatValues.SHADOWDMG = 0
	self.StatValues.NATUREDMG = 0
	self.StatValues.ARCANEDMG = 0
	self.StatValues.HOLYDMG = 0
	self.StatValues.HEAL = 0
	
	if TempSpellData.Healing then
		self.StatValues.HEAL = NextDamage
	else
		self.StatValues.DMG = NextDamage
		if MatchData(SpellSchool, "Fire") then
			self.StatValues.FIREDMG = NextDamage
		elseif MatchData(SpellSchool, "Frost") then
			self.StatValues.FROSTDMG = NextDamage
		elseif MatchData(SpellSchool, "Shadow") then
			self.StatValues.SHADOWDMG = NextDamage
		elseif MatchData(SpellSchool, "Nature") then
			self.StatValues.NATUREDMG = NextDamage
		elseif MatchData(SpellSchool, "Arcane") then
			self.StatValues.ARCANEDMG = NextDamage
		elseif MatchData(SpellSchool, "Holy") then
			self.StatValues.HOLYDMG = NextDamage
		end
	end
	
	self.StatValues.INT = NextSpellCrit * StatLogic:GetSpellCritFromInt(1, class, level)
	self.StatValues.INT = self.StatValues.INT + NextDamage * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")
	self.StatValues.INT = self.StatValues.INT * StatLogic:GetStatMod("MOD_INT")
	
	self.StatValues.SPI = NextDamage * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI")
	self.StatValues.SPI = self.StatValues.SPI * StatLogic:GetStatMod("MOD_SPI")
	
	self.StatValues.STA = NextDamage * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")
	self.StatValues.STA = self.StatValues.STA * StatLogic:GetStatMod("MOD_STA")
	
	-- Calculate to "SEP"
	--for stat in pairs(self.StatValues) do
	--	if stat ~= "DMG" then
	--		self.StatValues[stat] = self.StatValues[stat] / self.StatValues.DMG
	--	end
	--end
	
	--self.StatValues.DMG = 1
end

function DrDamageSet.prototype:Serialize()
	return {
		Type = "DrDamageSet",
		Name = self.Name,
		Enabled = self.Enabled,
		Spell = self.Spell,
		SpellRank = self.SpellRank,
		UpdateMode = self.UpdateMode,
		StatValues = self.StatValues,
	}
end

function DrDamageSet:Deserialize(t)
	return DrDamageSet:new(t.Name, t.Enabled, t.Spell, t.SpellRank, t.UpdateMode, t.StatValues)
end

ItemValue:RegisterSetType("DrDamageSet", DrDamageSet)
