--[[
	### ItemValue ###
	ItemValue StatSet Class
	
--]]

local class = select(2, UnitClass("player"))
if class ~= "ROGUE" and class ~= "WARRIOR" and class ~= "HUNTER" and class ~= "DRUID" and class ~= "PALADIN" and class ~= "SHAMAN" then return end

-- Libraries
local L = AceLibrary("AceLocale-2.2"):new("ItemValue_DrDamage")
local AceOO = AceLibrary("AceOO-2.0")
local ItemBonusLib = AceLibrary("ItemBonusLib-1.0")
local StatLogic = AceLibrary("StatLogic-1.0")

local class = select(2, UnitClass("player"))
local level = UnitLevel("player")

local Stats = {
	"STR", "AGI", "STA", "INT", "SPI",
	"DMG", "HOLYDMG",
	"CR_SPELLCRIT", "CR_SPELLHIT", "SPELLPEN",
	"ATTACKPOWER", "CR_CRIT", "CR_HIT",
}

local StatNames = {
	DMG					= L["Spelldamage"],
	HOLYDMG				= L["Holy Spelldamage"],
	CR_SPELLCRIT		= L["Spell Crit Rating"],
	CR_SPELLHIT			= L["Spell Hit Rating"],
	SPELLPEN			= L["Spell Penetration"],
	CR_SPELLHASTE		= L["Spell Haste Rating"],
}

local DrDamageSet = AceOO.Class()

local db
function DrDamageSet.LoadSets()
	db = ItemValue:AcquireDBNamespace("DrDamage_Melee")
	
	for SetName in pairs(db.profile) do
		if not ItemValue.SetByName[SetName] then
			ItemValue:RegisterSet(DrDamageSet:new(SetName))
		end
	end
end

local validate_SpellName = function(v)
	if (v and type(v) == "string" and v:len() > 3) then
		return true
	else
		ItemValue:Print(L["Error: Invalid spell name!"])
		return false
	end
end

function DrDamageSet.prototype:init(Name, Spell, UpdateMode, StatValues)
    DrDamageSet.super.prototype.init(self)
	
	if not db.profile[Name] then
		db.profile[Name] = {
			Spell = Spell or Name,
			UpdateMode = UpdateMode or L["Every Calculation"],
			StatValues = StatValues,
		}
	end
	
	local data = db.profile[Name]
	
	self.Name = Name
	
	self.Options = {
		type = "group",
		name = Name,
		desc = string.format(L["Settings for Set \"%s\"."], Name),
		args = {
			SpellName = {
				order = 10,
				type  = "text",
				name  = L["Spell"],
				desc  = L["Spell"],
				get   = function() return data.Spell end,
				set   = function(v) data.Spell = v self:UpdateValues() ItemValue:ClearCache() end,
				usage = L["<Spell Name>"],
				validate = validate_SpellName,
			},
			UpdateMode = {
				order = 12,
				type  = "text",
				name  = L["Update Mode"],
				desc  = L["Update Mode"],
				get   = function() return data.UpdateMode end,
				set   = function(v) data.UpdateMode = v self:UpdateValues() ItemValue:ClearCache() end,
				validate = {L["Every Calculation"], L["Saved Values"]}
			},
			UpdateValues = {
				order = 13,
				type  = "execute",
				name  = L["Update Values"],
				desc  = L["Updates saved values."],
				func  = function() self:UpdateValues() self:CalcValues() end,
				disabled = function() return data.UpdateMode ~= L["Saved Values"] end,
			},
			CopyToStatSet = {
				order = 14,
				type  = "text",
				name  = L["Copy to StatSet"],
				desc  = L["Creates a StatSet with identically StatValues."],
				usage = L["<Name>"],
				get   = false,
				set   = function(v)
					self:UpdateValues()
					local StatValues = {}
					for StatName, StatValue in pairs(data.StatValues) do
						StatValues[StatName] = StatValue
					end
					ItemValue.RegisterSet(ItemValue.classes.StatSet:new(v, StatValues))
				end,
				validate = function(v) return tostring(v) and string.len(v) > 0 end,
			},
			Values = {
				order = 15,
				type  = "group",
				name  = L["Values"],
				desc  = L["Displays current stat value data."],
				args  = {},
			},
		},
	}
	
	--self:UpdateValues()
	
	local get = function(k)
		if not data.StatValues then self:UpdateValues() end
		return data.StatValues[k] or 0
	end
	local set = function() end
	local hidden = function(k)
		if not (data.StatValues and data.StatValues[k]) or data.StatValues[k] == 0 then
			return true
		end
	end
	
	for i, iblname in pairs(Stats) do
		self.Options.args.Values.args[iblname] = {
			order	= i,
			type	= "text",
			name	= StatNames[iblname] or ItemBonusLib:GetBonusFriendlyName(iblname),
			desc	= StatNames[iblname] or ItemBonusLib:GetBonusFriendlyName(iblname),
			get		= get,
			set		= set,
			usage   = "<Read-only!>",
			hidden  = hidden,
			passValue = iblname,
		}
	end
end

function DrDamageSet.prototype:tostring()
	return "ItemValue DrDamageSet " .. self.Name
end

function DrDamageSet.prototype:OnDelete()
	db.profile[self.Name] = nil
end

function DrDamageSet.prototype:GetEquipValue(eq)
	if not (db.profile[self.Name].Spell) then return 0 end
	
	self:UpdateValues()
	
	local Points = 0
	
	if eq then
		for StatName, StatValue in pairs(db.profile[self.Name].StatValues) do
			if eq[StatName] and StatValue then
				Points = Points + (eq[StatName] * StatValue)
			end
		end
	end
	
	return Points
end

function DrDamageSet.prototype:GetItemValue(ItemString)
	return self:GetEquipValue(ItemBonusLib:ScanItem(ItemString, true))
end

function DrDamageSet.prototype:UpdateValues()
	if db.profile[self.Name].UpdateMode == L["Every Calculation"] then
		if not db.profile[self.Name].StatValues then
			db.profile[self.Name].StatValues = {}
		end
		
		self:CalcValues()
	end
end

-- From DrDamage
local function MatchData(data, ... )
	if not (data and ...) then
		return false
	end
	
	if type(data) == "table" then
		for _, name in ipairs( data ) do
			for i=1, select('#', ...) do
				if name == select(i, ...) then
					return true
				end
			end
		end
	else
		for i=1, select('#', ...) do
			if data == select(i, ...) then
				return true
			end
		end
	end
	
	return false
end

function DrDamageSet.prototype:CalcValues()
	local Spell = db.profile[self.Name].Spell
	if not validate_SpellName(Spell) then return end
	
	local TempSpellData
	if Spell == "Attack" then
		TempSpellData = select(2, DrDamage:MeleeCalc({0}, Spell, true, true))
	else
		local TableSpell = DrDamage.spellInfo[Spell] and DrDamage.spellInfo[Spell][#DrDamage.spellInfo[Spell]]
		if TableSpell then
			TempSpellData = select(2, DrDamage:MeleeCalc(TableSpell, Spell, true, true))
		else
			ItemValue:Print(string.format(L["Error: Couldn't get TableSpell for |cffffff78%s|r from DrDamage!"], Spell))
			return
		end
	end
	
	if not TempSpellData then
		ItemValue:Print(string.format(L["Error: Couldn't get informations about |cffffff78%s|r from DrDamage!"], Spell))
		return
	end
	
	local NextCrit = TempSpellData.NextCrit or 0
	local NextHit = TempSpellData.NextHit or 0
	local NextAP = (TempSpellData.NextAP * 0.01 * StatLogic:GetStatMod("MOD_AP")) or 0
	
	for k in pairs(db.profile[self.Name].StatValues) do
		db.profile[self.Name].StatValues[k] = nil
	end
	
	db.profile[self.Name].StatValues.STR = NextAP * StatLogic:GetStatMod("MOD_STR")
	db.profile[self.Name].StatValues.ATTACKPOWER = NextAP
	db.profile[self.Name].StatValues.CR_CRIT = NextCrit * DrDamage:GetRating("Crit", 1, true)
	db.profile[self.Name].StatValues.CR_HIT = NextHit * DrDamage:GetRating("Hit", 1, true)
	db.profile[self.Name].StatValues.AGI = (NextCrit/40.4 + NextAP) * StatLogic:GetStatMod("MOD_AGI")
end

function DrDamageSet.prototype:Serialize()
	return {
		Type = "DrDamageSet",
		Name = self.Name,
		Spell = self.Spell,
		UpdateMode = self.UpdateMode,
		StatValues = self.StatValues,
	}
end

function DrDamageSet:Deserialize(t)
	return DrDamageSet:new(t.Name, t.Spell, t.UpdateMode, t.StatValues)
end

ItemValue:RegisterSetType("DrDamage_Melee", DrDamageSet)
